@@ -17,16 +17,21 @@ import uuid
 import warnings
 
 from datetime import datetime
-from functools import wraps
 from pprint import pformat
 from types import FunctionType
 
+if sys.version_info >= (3, 2):
+    import contextlib
+else:
+    import contextlib2 as contextlib
+
 import raven
 from raven.conf import defaults
 from raven.conf.remote import RemoteConfig
 from raven.context import Context
 from raven.exceptions import APIError, RateLimited
-from raven.utils import six, json, get_versions, get_auth_header, merge_dicts
+from raven.utils import json, get_versions, get_auth_header, merge_dicts
+from raven._compat import text_type, iteritems
 from raven.utils.encoding import to_unicode
 from raven.utils.serializer import transform
 from raven.utils.stacks import get_stack_info, iter_stack_frames, get_culprit
@@ -141,7 +146,7 @@ class Client(object):
 
         self.include_paths = set(o.get('include_paths') or [])
         self.exclude_paths = set(o.get('exclude_paths') or [])
-        self.name = six.text_type(o.get('name') or o.get('machine') or defaults.NAME)
+        self.name = text_type(o.get('name') or o.get('machine') or defaults.NAME)
         self.auto_log_stacks = bool(
             o.get('auto_log_stacks') or defaults.AUTO_LOG_STACKS)
         self.capture_locals = bool(
@@ -161,6 +166,7 @@ class Client(object):
             context = {'sys.argv': sys.argv[:]}
         self.extra = context
         self.tags = o.get('tags') or {}
+        self.environment = o.get('environment') or None
         self.release = o.get('release') or os.environ.get('HEROKU_SLUG_COMMIT')
 
         self.module_cache = ModuleProxyCache()
@@ -247,7 +253,8 @@ class Client(object):
         >>> result = client.capture(**kwargs)
         >>> ident = client.get_ident(result)
         """
-        warnings.warn('Client.get_ident is deprecated. The event ID is now returned as the result of capture.',
+        warnings.warn('Client.get_ident is deprecated. The event ID is now '
+                      'returned as the result of capture.',
                       DeprecationWarning)
         return result
 
@@ -301,7 +308,7 @@ class Client(object):
         if data.get('culprit'):
             culprit = data['culprit']
 
-        for k, v in six.iteritems(result):
+        for k, v in iteritems(result):
             if k not in data:
                 data[k] = v
 
@@ -364,6 +371,9 @@ class Client(object):
         if self.release is not None:
             data['release'] = self.release
 
+        if self.environment is not None:
+            data['environment'] = self.environment
+
         data['tags'] = merge_dicts(self.tags, data['tags'], tags)
         data['extra'] = merge_dicts(self.extra, data['extra'], extra)
 
@@ -386,11 +396,11 @@ class Client(object):
             data['message'] = kwargs.get('message', handler.to_string(data))
 
         # tags should only be key=>u'value'
-        for key, value in six.iteritems(data['tags']):
+        for key, value in iteritems(data['tags']):
             data['tags'][key] = to_unicode(value)
 
         # extra data can be any arbitrary value
-        for k, v in six.iteritems(data['extra']):
+        for k, v in iteritems(data['extra']):
             data['extra'][k] = self.transform(v)
 
         # It's important date is added **after** we serialize
@@ -548,7 +558,8 @@ class Client(object):
             if isinstance(exc, RateLimited):
                 retry_after = exc.retry_after
             self.error_logger.error(
-                'Sentry responded with an API error: %s(%s)', type(exc).__name__, exc.message)
+                'Sentry responded with an API error: %s(%s)',
+                type(exc).__name__, exc.message)
         else:
             self.error_logger.error(
                 'Sentry responded with an error: %s (url: %s)\n%s',
@@ -569,10 +580,10 @@ class Client(object):
         if 'exception' in data and 'stacktrace' in data['exception']['values'][0]:
             # try to reconstruct a reasonable version of the exception
             for frame in data['exception']['values'][0]['stacktrace']['frames']:
-                output.append('  File "%(filename)s", line %(lineno)s, in %(function)s' % {
-                    'filename': frame['filename'],
+                output.append('  File "%(fn)s", line %(lineno)s, in %(func)s' % {
+                    'fn': frame['filename'],
                     'lineno': frame['lineno'],
-                    'function': frame['function'],
+                    'func': frame['function'],
                 })
 
         self.uncaught_logger.error(output)
@@ -689,10 +700,11 @@ class Client(object):
         return self.capture(
             'raven.events.Exception', exc_info=exc_info, **kwargs)
 
-    def capture_exceptions(self, function_or_exceptions, **kwargs):
+    def capture_exceptions(self, function_or_exceptions=None, **kwargs):
         """
-        Wrap a function in try/except and automatically call ``.captureException``
-        if it raises an exception, then the exception is reraised.
+        Wrap a function or code block in try/except and automatically call
+        ``.captureException`` if it raises an exception, then the exception
+        is reraised.
 
         By default, it will capture ``Exception``
 
@@ -700,28 +712,41 @@ class Client(object):
         >>> def foo():
         >>>     raise Exception()
 
+        >>> with client.capture_exceptions():
+        >>>    raise Exception()
+
         You can also specify exceptions to be caught specifically
 
         >>> @client.capture_exceptions((IOError, LookupError))
         >>> def bar():
         >>>     ...
 
+        >>> with client.capture_exceptions((IOError, LookupError)):
+        >>>    ...
+
         ``kwargs`` are passed through to ``.captureException``.
         """
-        def make_decorator(exceptions):
-            def decorator(func):
-                @wraps(func)
-                def wrapper(*funcargs, **funckwargs):
-                    try:
-                        return func(*funcargs, **funckwargs)
-                    except exceptions:
-                        self.captureException(**kwargs)
-                        raise
-                return wrapper
-            return decorator
+        function = None
+        exceptions = (Exception,)
         if isinstance(function_or_exceptions, FunctionType):
-            return make_decorator((Exception,))(function_or_exceptions)
-        return make_decorator(function_or_exceptions)
+            function = function_or_exceptions
+        elif function_or_exceptions is not None:
+            exceptions = function_or_exceptions
+
+        # In python3.2 contextmanager acts both as contextmanager and decorator
+        @contextlib.contextmanager
+        def make_decorator(exceptions):
+            try:
+                yield
+            except exceptions:
+                self.captureException(**kwargs)
+                raise
+
+        decorator = make_decorator(exceptions)
+
+        if function:
+            return decorator(function)
+        return decorator
 
     def captureQuery(self, query, params=(), engine=None, **kwargs):
         """
